/*
V 1.3.4
Version of Grid Traffic Simulation on March 9, 2000
Copyright (c) by Lei Zhou.
 
   .One-way & one-lane grid traffic situation with periodical boundary conditions 
   .Turning randomly at intersections with certain turning rate
   .Make turnning decision just once
   .Using waitTime field instead of turnMade field in Vehicle class, 
 every car have to stay at intersections for atleast 2 seconds, they make 
 turning decision and change their directions if applying at the 2nd time step.    
   .carColor is an int field in Vehicle class, change to Color variables 
 in TrafficGraphic class
   .Using swing package
   .Using TrafficGraphic, put the Street panel and TrafficGrapphic panel (overlap)
 into a scrollPane 

   .add arrow to indicate the directions of links
   .Write the characters to output file every time step.
	.if the waitTime=MaxWaitTime of a vehicle at some intersections, 
 try to make another turn (MaxWaitTime is finite)
 	.flow rate is # of cars passing all the links in one time step, 
 watching point is the most front of link
   
   
*/

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;


public class GridTraffic extends JFrame
    implements TrafficConstants {

    private TrafficGraphic myTrafficGraphic;
    private JTextField tfCarNo, tfDensity, tfTurn,tfFlow,tfSpeed;
    private JRadioButton beginButton,runButton,stopButton;
    private JSlider densitySlider, pturnSlider;
    private float density, pturn;

    public GridTraffic(String title){
        super(title);
        Container c= getContentPane();
        c.setLayout(new BorderLayout());

        ImageIcon dukeIcon = new ImageIcon("images/duke2.gif");
        ImageIcon dukeWaveIcon = new ImageIcon("images/dukeWave.gif");
        ImageIcon dukeWaveRedIcon = new ImageIcon("images/dukeWaveRed.gif");
        
        JPanel controlGroup = new JPanel();
        controlGroup.setBackground(new Color(255,228,212));
        controlGroup.setAlignmentX(LEFT_ALIGNMENT);
        controlGroup.setBorder(new CompoundBorder(
               new TitledBorder("Image ToggleButtons"),
               new EmptyBorder(5,5,5,5)));

        ButtonGroup buttonGroup = new ButtonGroup(); 

        beginButton= new JRadioButton("Begin",dukeIcon);
        beginButton.setSelectedIcon(dukeWaveIcon);
        beginButton.setPressedIcon(dukeWaveRedIcon);
        beginButton.setRolloverIcon(dukeWaveRedIcon);
        beginButton.setToolTipText("Reset the System");
        buttonGroup.add(beginButton);
        controlGroup.add(beginButton);
       
        runButton = new JRadioButton("Run",dukeIcon);
        runButton.setSelectedIcon(dukeWaveIcon);
        runButton.setPressedIcon(dukeWaveRedIcon);
        runButton.setRolloverIcon(dukeWaveRedIcon);
        runButton.setToolTipText("Click to Run");
        buttonGroup.add(runButton);
        controlGroup.add(runButton);
 
        stopButton = new JRadioButton("Stop",dukeIcon);
        stopButton.setSelectedIcon(dukeWaveIcon);
        stopButton.setPressedIcon(dukeWaveRedIcon);
        stopButton.setRolloverIcon(dukeWaveRedIcon);
        stopButton.setToolTipText("Stop the Animation");
        buttonGroup.add(stopButton);
        controlGroup.add(stopButton);
                                                  
        JPanel densityPanel = new JPanel();
        densityPanel.setBackground(new Color(255,228,212));
        densityPanel.setLayout(new BoxLayout(densityPanel, BoxLayout.Y_AXIS));
        densityPanel.setBorder(new TitledBorder("Density"));  
        densitySlider = new JSlider(SwingConstants.HORIZONTAL,0,200,100);
        densitySlider.setMajorTickSpacing(20);
        densitySlider.setPaintTicks(true);
        densitySlider.setToolTipText("Pick Density"); 
        densitySlider.setPreferredSize(new Dimension(80,30));
        densitySlider.addChangeListener(
           new ChangeListener(){
              public void stateChanged(ChangeEvent e)
              {
                  density = (float)densitySlider.getValue()/(float)200.0;
                  
              }
           }
        );     
        density = (float)densitySlider.getValue()/(float)200.0;    
        densityPanel.add(densitySlider);

        
        JPanel pturnPanel = new JPanel();
        pturnPanel.setBackground(new Color(255,228,212));
        pturnPanel.setLayout(new BoxLayout(pturnPanel, BoxLayout.Y_AXIS));
        pturnPanel.setBorder(new TitledBorder("Turning"));  
        pturnSlider = new JSlider(SwingConstants.HORIZONTAL,0,200,100);
        pturnSlider.setMajorTickSpacing(20);
        pturnSlider.setPaintTicks(true);
        pturnSlider.setToolTipText("Pick Turning Possibility 0--1");
        pturnSlider.setPreferredSize(new Dimension(80,30)); 
        pturnSlider.addChangeListener(
           new ChangeListener(){
              public void stateChanged(ChangeEvent e)
              {
                  pturn = (float)pturnSlider.getValue()/(float)200.0;
                  
              }
           }
        );     
        pturn = (float)pturnSlider.getValue()/(float)200.0;    
        pturnPanel.add(pturnSlider);
         
        controlGroup.add(beginButton);        
        controlGroup.add(runButton);
        controlGroup.add(stopButton); 
        controlGroup.add(densityPanel);
        controlGroup.add(pturnPanel);
        
                
        JPanel textGroup = new JPanel();  
        textGroup.setBackground(Color.pink);
        
        tfCarNo = new JTextField("",6);
        JPanel carNoPanel = new JPanel();
        carNoPanel.setLayout(new BoxLayout(carNoPanel,BoxLayout.Y_AXIS));
        carNoPanel.setBorder(new TitledBorder("Car No"));
        carNoPanel.setBackground(Color.pink);
        carNoPanel.add(tfCarNo);

        tfDensity = new JTextField(Float.toString(density),6);
        JPanel tfDensityPanel = new JPanel();
        tfDensityPanel.setLayout(new BoxLayout(tfDensityPanel,BoxLayout.Y_AXIS));
        tfDensityPanel.setBorder(new TitledBorder("Density"));
        tfDensityPanel.setBackground(Color.pink);
        tfDensityPanel.add(tfDensity);

        tfTurn = new JTextField(Float.toString(pturn),6); 
        JPanel tfTurnPanel = new JPanel();
        tfTurnPanel.setLayout(new BoxLayout(tfTurnPanel,BoxLayout.Y_AXIS));
        tfTurnPanel.setBorder(new TitledBorder("Turn Rate"));
        tfTurnPanel.setBackground(Color.pink);
        tfTurnPanel.add(tfTurn);

        JTextField tfSpeed = new JTextField("0.0",6); 
        JPanel tfSpeedPanel = new JPanel();
        tfSpeedPanel.setLayout(new BoxLayout(tfSpeedPanel,BoxLayout.Y_AXIS));
        tfSpeedPanel.setBorder(new TitledBorder("Average V"));
        tfSpeedPanel.setBackground(Color.pink);
        tfSpeedPanel.add(tfSpeed);

        tfFlow = new JTextField("0.0",6); 
        JPanel tfFlowPanel = new JPanel();
        tfFlowPanel.setLayout(new BoxLayout(tfFlowPanel,BoxLayout.Y_AXIS));
        tfFlowPanel.setBorder(new TitledBorder("Flow Rate"));
        tfFlowPanel.setBackground(Color.pink);
        tfFlowPanel.add(tfFlow);

        JPanel tfTimePanel = new JPanel();
        JTextField tfTime = new JTextField("0",6);
        tfTimePanel.setLayout(new BoxLayout(tfTimePanel,BoxLayout.Y_AXIS));
        tfTimePanel.setBorder(new TitledBorder("Time"));
        tfTimePanel.setBackground(Color.pink);
        tfTimePanel.add(tfTime);
              
        textGroup.add(carNoPanel);
        textGroup.add(tfDensityPanel);                     
        textGroup.add(tfTurnPanel);       
        textGroup.add(tfSpeedPanel);
        textGroup.add(tfFlowPanel);
        textGroup.add(tfTimePanel);	           
        
        JPanel controls = new JPanel();
        controls.setLayout(new GridLayout(2,1));
        controls.add(controlGroup);
        controls.add(textGroup); 
       
        myTrafficGraphic = new TrafficGraphic(beginButton,runButton, stopButton,
            tfCarNo,tfDensity,tfTurn,tfSpeed,tfFlow,tfTime,
            densitySlider,pturnSlider);
        
        myTrafficGraphic.setMaximumSize(new Dimension(1020,820));
       
        Street myStreetPanel = new Street();
        myStreetPanel.setMaximumSize(new Dimension(1020,820));
        myStreetPanel.setLayout(new GridLayout(1,1));
        myStreetPanel.add(myTrafficGraphic);          
        Border raisedbevel = BorderFactory.createRaisedBevelBorder();
        Border loweredbevel = BorderFactory.createLoweredBevelBorder();
        myStreetPanel.setBorder(
            BorderFactory.createCompoundBorder(raisedbevel, loweredbevel));
     
        JScrollPane scroller = new JScrollPane();
        scroller.setViewportView(myStreetPanel);
        scroller.setMaximumSize(new Dimension(1020,820));
     
     
        c.add(controls,"North");
        c.add(scroller, "Center" );
        pack();

           
    /* now I set myTraffic is a public field of TrafficCanvas, it seems that
       myTraffic can not be created in TrafficCanvas class
    */    
        int n = myTrafficGraphic.getCarNo();
        tfCarNo.setText(Integer.toString(n)); 
        density = (n/(float)siteNo);
        String strDen = Float.toString(density);
        if(strDen.length()>=8)
            tfDensity.setText(strDen.substring(0,8));
        else
            tfDensity.setText(strDen);       
    }
   
    public static void main(String args[]) {

        JFrame myGridTraffic = new GridTraffic("Simulation of City Traffic");
        myGridTraffic.addWindowListener(
            new WindowAdapter(){
                public void windowClosing(WindowEvent e)
                {
                    System.exit(0);
                }
            }
        );//WindowAdapter is an anonymous inner class
        myGridTraffic.setSize(700,600);
        myGridTraffic.show();
   }
}
