import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class TrafficGraphic extends JPanel 
    implements ActionListener, TrafficConstants{

       
    private Traffic myTraffic; 
    private int[][] params;
    private Color[] colors;
    private int n, velocitySum;
    private long time;
       
    private float density, pturn;
    private JRadioButton beginButton,runButton,stopButton;
    private JSlider densitySlider, pturnSlider;
    private JTextField tfCarNo,tfDensity,tfTurn,tfSpeed,tfFlow,tfTime;
    private Timer timer;
    
   
    public TrafficGraphic (JRadioButton beginButton,JRadioButton runButton, 
         JRadioButton stopButton, JTextField tfCarNo, JTextField tfDensity, 
         JTextField tfTurn, JTextField tfSpeed,JTextField tfFlow, JTextField tfTime, 
         JSlider densitySlider, JSlider pturnSlider){

        super();    
        setOpaque(false); 
        
        this.runButton = runButton;
        this.runButton.addActionListener(this);
        this.beginButton = beginButton;
        this.beginButton.addActionListener(this);
        this.stopButton = stopButton;
        this.stopButton.addActionListener(this);

        this.densitySlider = densitySlider;
        this.pturnSlider = pturnSlider;
        this.tfCarNo = tfCarNo;
        this.tfDensity = tfDensity;
        this.tfTurn = tfTurn;
        this.tfSpeed = tfSpeed;
        this.tfFlow = tfFlow;
        this.tfTime = tfTime;      
            
        this.density = (float)this.densitySlider.getValue()/(float)200.0;
        this.pturn = (float)this.pturnSlider.getValue()/(float)200.0;
        myTraffic = new Traffic(density,pturn);
        n = myTraffic.computeForDraw();
        params = new int[n][6];
        colors = new Color[n];
        velocitySum = 0;
        timer = new Timer(ONE_SECOND,this);
        time = 0;
   }

  
    public int getCarNo(){
        return n;
    }
    
    public void paintComponent(Graphics g) {
   
   
      //the size of system is 1000*800, 20*20 pixel per site
      getParams();
      for(int i=0; i<n;i++){
          g.setColor(colors[i]);
          g.fill3DRect(params[i][0]-4,params[i][1]-4,params[i][2],params[i][3],true);
      }
    }
   
   public void getParams()
   {  
       velocitySum = 0;
       for(int i=0; i<n;i++)
           for(int j=0;j<6;j++)
               params[i][j] = myTraffic.getDraw(i,j); 

       for(int i=0;i<n;i++)
           velocitySum +=params[i][5];

    for(int i=0;i<n;i++)
      switch(params[i][4]){
      case 1:
         {
            colors[i] = Color.blue;
            break;
         }        
      case 2:
         {
            colors[i] = Color.red;
            break;
         }
      case 3:
         {
            colors[i] = Color.yellow;
            break;
         }
      case 4:
         {
            colors[i] = Color.black;
            break;
         }
      case 5:
         {
            colors[i] = Color.cyan;
            break;
         }
      case 6:
         {
            colors[i] = Color.orange;
            break;
         }
      case 7:
         {
            colors[i] = Color.green;
            break;
         }
      case 8:
         {
            colors[i] = Color.pink;
            break;
         }
     default:
       colors[i] = Color.blue;
      }
   } 
   
   public Dimension getPreferredSize()
   {
       return new Dimension(1020,820);
   }

   public void actionPerformed(ActionEvent ae)
   {
       String strDen,strTurn, strSpd, strFlow;

       if(ae.getSource()==beginButton){  //reset the traffic
           this.density = (float)this.densitySlider.getValue()/(float)200.0;
           this.pturn = (float)this.pturnSlider.getValue()/(float)200.0;
           myTraffic = new Traffic(density,pturn);
           n = myTraffic.computeForDraw();
           velocitySum = 0;
           time = 0;
           this.tfCarNo.setText(Integer.toString(n));

           strDen = Float.toString(n/(float)siteNo);          
           if(strDen.length()>= 8)
               this.tfDensity.setText(strDen.substring(0,8));
           else 
               this.tfDensity.setText(strDen);
 
           strTurn = Float.toString(pturn);
           if(strTurn.length()>= 8)
               this.tfTurn.setText(strTurn.substring(0,8));
           else
               this.tfTurn.setText(strTurn);
 
           strSpd = Float.toString((float)velocitySum/(float)n);
           if(strSpd.length() >= 8)
               this.tfSpeed.setText(strSpd.substring(0,8));
           else
               this.tfSpeed.setText(strSpd);

           strFlow = Float.toString(myTraffic.getFlow());
           if(strFlow.length() >= 8)
               this.tfFlow.setText(strFlow.substring(0,8));
           else
               this.tfFlow.setText(strFlow);
           this.tfTime.setText(Long.toString(time));
           params = new int[n][6];
           colors = new Color[n];
           repaint();  
       }

       else if(ae.getSource()==timer){
           myTraffic.updateTraffic();
           int carNo = myTraffic.computeForDraw();
           this.tfCarNo.setText(Integer.toString(carNo));
           repaint(); 
			  float avgV = (float)velocitySum/(float)n;
           strSpd = Float.toString(avgV);
           if(strSpd.length() >= 8)
               this.tfSpeed.setText(strSpd.substring(0,8));
           else
               this.tfSpeed.setText(strSpd);

           strFlow = Float.toString((float)velocitySum/(float)siteNo);
           if(strFlow.length() >= 8)
               this.tfFlow.setText(strFlow.substring(0,8));
           else
               this.tfFlow.setText(strFlow);

           this.tfTime.setText(Long.toString(++time));    
			  
       } 
 
       else if(ae.getSource()==runButton)
           timer.start();

       else if(ae.getSource()==stopButton)
           timer.stop();            
       
   }
}

    
